# -*- coding: utf-8 -*-

"""Main module."""

import torch
from .inference import segment_resection


def resseg(input_path, output_path, window_size, window_border, batch_size):
    repo = 'fepegar/resseg'
    model_name = 'resseg'
    print(torch.hub.help(repo, model_name))
    model = torch.hub.load(repo, model_name, pretrained=True)
    segment_resection(
        input_path,
        model,
        window_size,
        window_border,
        output_path,
        batch_size=batch_size,
    )



# -*- coding: utf-8 -*-

"""Top-level package for resseg."""

__author__ = """Fernando Perez-Garcia"""
__email__ = 'fernando.perezgarcia.17@ucl.ac.uk'
__version__ = '0.1.0'

from .resseg import resseg



import numpy as np
import nibabel as nib
from .nifti import load, save


def flip_lr(input_path, output_path):
    # Asume R-- or L-- (e.g. RAS or LPS)
    nii = load(input_path, mmap=False)
    data = nii.get_data()
    data = data[::-1, ...]
    save(data, affine=nii.affine, path=output_path)


def mean_image(input_paths, output_path):
    niis = [load(fp) for fp in input_paths]
    arrays = [nii.get_data() for nii in niis]
    mean = np.mean(arrays, axis=0)
    save(mean, path=output_path, affine=niis[0].affine)


def binarize_probabilities(input_path, output_path):
    nii = load(input_path, mmap=False)
    data = nii.get_data() > 0.5
    data = data.astype(np.uint8)
    save(data, affine=nii.affine, path=output_path)



# -*- coding: utf-8 -*-

"""Console script for resseg."""
import sys
import click

INPUT_FILE_TYPE = click.Path(exists=True, dir_okay=False)
OUTPUT_FILE_TYPE = click.Path(dir_okay=False)

@click.command()
@click.argument('input-path', type=INPUT_FILE_TYPE)
@click.argument('output-path', type=OUTPUT_FILE_TYPE)
@click.option(
    '--window_size', '-w',
    type=int,
    default=128,
    show_default=True,
)
@click.option(
    '--window_border', '-c',
    type=int,
    default=1,
    show_default=True,
)
@click.option(
    '--batch_size', '-b',
    type=int,
    default=1,
    show_default=True,
)
def main(input_path, output_path, window_size, window_border, batch_size):
    """Console script for resseg."""
    """
    For now, the image is assumed to have been preprocessed:
    1) Reg to MNI, RAS 1 mm iso
    2) Brain segmented
    3) Bias correction
    4) Histogram standardization
    5) Z-normalization
    """
    from resseg import resseg
    resseg(
        input_path,
        output_path,
        window_size,
        window_border,
        batch_size,
    )
    return 0


if __name__ == "__main__":
    # pylint: disable=no-value-for-parameter
    sys.exit(main())  # pragma: no cover



from tempfile import NamedTemporaryFile

import numpy as np
from tqdm import tqdm
import torch
from torch.utils.data import DataLoader

from .grid_sampler import GridSampler
from .grid_aggregator import GridAggregator
from .postprocessing import binarize_probabilities, flip_lr, mean_image


def to_tuple(value):
    try:
        iter(value)
    except TypeError:
        value = 3 * (value,)
    return value


def get_device():
    # pylint: disable=no-member
    return torch.device('cuda' if torch.cuda.is_available() else 'cpu')


def segment_resection(
        input_path,
        model,
        window_size,
        window_border,
        output_path,
        batch_size=None,
        show_progress=True,
        flip=True,
        binarize=True,
        ):

    run_inference(
        input_path,
        model,
        output_path=output_path,
        window_size=window_size,
        window_border=window_border,
        batch_size=batch_size,
        show_progress=show_progress,
    )

    if flip:
        with NamedTemporaryFile(suffix='.nii') as output_temp:
            with NamedTemporaryFile(suffix='.nii') as input_temp:
                flip_lr(input_path, input_temp.name)
                run_inference(
                    input_temp.name,
                    model,
                    output_path=output_temp.name,
                    window_size=window_size,
                    window_border=window_border,
                    batch_size=batch_size,
                    show_progress=show_progress,
                )
            flip_lr(output_temp.name, output_temp.name)
            paths = output_path, output_temp.name
            mean_image(paths, output_path)

    if binarize:
        binarize_probabilities(output_path, output_path)


def run_inference(
        image_path,
        model,
        output_path,
        window_size,
        window_border=None,
        batch_size=None,
        show_progress=True,
        ):
    window_border = 1 if window_border is None else window_border
    batch_size = 1 if batch_size is None else batch_size
    window_size = to_tuple(window_size)
    window_border = to_tuple(window_border)

    sampler = GridSampler(
        image_path, window_size, window_border, dtype=np.float32)
    aggregator = GridAggregator(image_path, window_border)
    loader = DataLoader(sampler, batch_size=batch_size)

    device = get_device()
    print('Using device', device)

    model.to(device)
    model.eval()

    with torch.no_grad():
        progress = tqdm(loader) if show_progress else loader
        for batch in progress:
            input_tensor = batch['image'].to(device)
            locations = batch['location']
            logits = model(input_tensor)
            probabilities = logits.softmax(dim=1)
            foreground = probabilities[:, 1:, ...]
            outputs = foreground
            aggregator.add_batch(outputs, locations)

    aggregator.save_current_image(
        output_path,
        output_probabilities=True,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=7.0',
    'nibabel',
    'numpy',
    'torch>=1.1',
    'tqdm',
    'unet',
    # 'utils @ https://github.com/fepegar/utils/archive/master.zip#egg=utils'
]

setup_requirements = []

test_requirements = []


setup(
    author="Fernando Perez-Garcia",
    author_email='fernando.perezgarcia.17@ucl.ac.uk',
    python_requires='>2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    description="Automatic segmentation of epilepsy neurosurgery resection cavity.",
    entry_points={
        'console_scripts': [
            'resseg=resseg.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='resseg',
    name='resseg',
    packages=find_packages(include=['resseg', 'resseg.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/fepegar/resseg',
    version='0.1.0',
    zip_safe=False,
)



from pathlib import Path
from typing import Union, Optional
import numpy as np
import nibabel as nib


def load(
        path: Union[str, Path],
        mmap: bool = True,
        ) -> nib.Nifti1Image:
    nii = nib.load(str(path), mmap=mmap)
    return nii

def save(
        data: np.ndarray,
        path: Union[str, Path],
        affine: Optional[np.ndarray] = None,
        rgb: bool = False,
        ) -> None:
    nii = nib.Nifti1Image(data, affine)
    nii.header['qform_code'] = 1
    nii.header['sform_code'] = 0
    if rgb:
        nii.header.set_intent('vector')
    nib.save(nii, str(path))


def get_data(path: Union[str, Path]) -> np.ndarray:
    data = load(path).get_data()
    if isinstance(data, np.memmap):
        data = np.array(data)
    return data


read = load
write = save
